// Earthworm Quest
// A nibbles clone for the ITG challenge at Birdie 17.
// By lft.

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <err.h>
#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

// Include the graphics data.

#include "soil.h"
#include "stones.h"
#include "flower.h"
#include "font.h"
#include "header.h"

// The window dimensions. Must me 4:3.

#define WIDTH 800
#define HEIGHT 600

// The playfield.

char board[] =
	"################"
	"#..............#"
	"#..............#"
	"#..............#"
	"#..............#"
	"#.....#........#"
	"#..............#"
	"#..............#"
	"#..............#"
	"#..............#"
	"#.......##.....#"
	"#..............#"
	"#..............#"
	"#..............#"
	"#..............#"
	"################"
;

// Some parameters that can be adjusted for performance/playability tweaking.

#define MAXSEG 300
#define NPARTICLE 80

// Textures.

enum {
	T_GROUND,
	T_STONES,
	T_FONT,
	T_HEADER,
	T_FLOWER
};

// Data structures to keep track of the worm.

struct segment {
	double		x, z;		// [-2, 2]
};

struct worm {
	struct segment	segs[MAXSEG];
	int		length;		// [30, MAXSEG]
	int		growto;		// [30, MAXSEG]
	int		angle;		// [0, 360]
	double		speed;		// Starts out at .018
	int		wobblecount;	// A simple up-counter used for animating the worm.
} worm;

// Data structures to keep track of the explosion animation.

struct blamparticle {
	double		x, y, z;
	double		dx, dy, dz;
} blamparticle[NPARTICLE];

// Data structures to keep track of the game status.

double foodpos[2];	// X and Z coordinates, [-2, 2]
int growtimer;
int score;
int gameover;

// OpenGL specific data.

GLUquadric *quad;

// Scale the two-dimensional vector v down (or up) to unit length.

void normalize2(double *v) {
	double length;

	length = hypot(v[0], v[1]);
	v[0] /= length;
	v[1] /= length;
}

// Draw a frame, based on current game and animation status.

void drawframe() {
	int i, y, x;
	GLfloat fv[4];
	double h, size;
	double eye[2], direction[2], lookat[2];
	double length;

	// Clear the screen.

	glClearColor(.1, .1, .3, 0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// Prepare to draw the ground.

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(30, (double) WIDTH / HEIGHT, 1, 200);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);
	glDisable(GL_CULL_FACE);
	glDisable(GL_BLEND);

	fv[0] = 0;
	fv[1] = 1;
	fv[2] = 1;
	fv[3] = 0;
	glLightfv(GL_LIGHT0, GL_POSITION, fv);
	fv[0] = .5;
	fv[1] = .4;
	fv[2] = .3;
	fv[3] = 0;
	glLightfv(GL_LIGHT0, GL_DIFFUSE, fv);
	fv[0] = .5;
	fv[1] = .5;
	fv[2] = .5;
	fv[3] = 0;
	glLightfv(GL_LIGHT0, GL_SPECULAR, fv);

	direction[0] = worm.segs[4].x - worm.segs[5].x;
	direction[1] = worm.segs[4].z - worm.segs[5].z;
	normalize2(direction);

	eye[0] = worm.segs[4].x - direction[0] * 3;
	eye[1] = worm.segs[4].z - direction[1] * 3;

	lookat[0] = worm.segs[4].x + direction[0] * 3;
	lookat[1] = worm.segs[4].z + direction[1] * 3;

	gluLookAt(eye[0], .8, eye[1], lookat[0], .1, lookat[1], 0, 1, 0);

	// Draw the ground (brown circle and grass above it).

	fv[0] = .2;
	fv[1] = 0;
	fv[2] = 0;
	fv[3] = 0;
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fv);
	glDisable(GL_TEXTURE_2D);
	glNormal3d(0, 1, 0);
	glBegin(GL_TRIANGLE_FAN);
	glVertex3d(0, 0, 0);
	for(i = 0; i <= 128; i++) {
		glVertex3d(5 * cos(i * M_PI * 2 / 128), -.1, 5 * sin(i * M_PI * 2 / 128));
	}
	glEnd();

	fv[0] = 1;
	fv[1] = 1;
	fv[2] = 1;
	fv[3] = 0;
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fv);
	fv[0] = 0;
	fv[1] = 0;
	fv[2] = 0;
	fv[3] = 0;
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, fv);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, T_GROUND);

	glNormal3d(0, 1, 0);
	glBegin(GL_QUADS);
	glTexCoord2d(-2, -2);
	glVertex3d(-2, 0, -2);
	glTexCoord2d(+2, -2);
	glVertex3d(+2, 0, -2);
	glTexCoord2d(+2, +2);
	glVertex3d(+2, 0, +2);
	glTexCoord2d(-2, +2);
	glVertex3d(-2, 0, +2);
	glEnd();

	// Draw the blocks of stone.

	h = .06;
	glBindTexture(GL_TEXTURE_2D, T_STONES);
	for(y = 0; y < 16; y++) {
		for(x = 0; x < 16; x++) {
			double cx, cy;	// Center of block.
			double sz;	// Size of block (half the length of the side).

			cx = (-7.5 + x) * 2.0 / 8;
			cy = (-7.5 + y) * 2.0 / 8;
			sz = 2.0 / 8 / 2;

			if(board[y * 16 + x] == '#') {
				glBegin(GL_QUADS);

				fv[0] = 1;
				fv[1] = 1;
				fv[2] = 1;
				fv[3] = 0;
				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fv);
				glNormal3d(0, 1, 0);
				glTexCoord2d(cx - sz, cy - sz);
				glVertex3d(cx - sz, h, cy - sz);
				glTexCoord2d(cx + sz, cy - sz);
				glVertex3d(cx + sz, h, cy - sz);
				glTexCoord2d(cx + sz, cy + sz);
				glVertex3d(cx + sz, h, cy + sz);
				glTexCoord2d(cx - sz, cy + sz);
				glVertex3d(cx - sz, h, cy + sz);

				fv[0] = .5;
				fv[1] = .5;
				fv[2] = .5;
				fv[3] = 0;
				glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fv);
				glNormal3d(0, 0, -1);
				glTexCoord2d(cx - sz, cy - sz);
				glVertex3d(cx - sz, h, cy - sz);
				glTexCoord2d(cx + sz, cy - sz);
				glVertex3d(cx + sz, h, cy - sz);
				glTexCoord2d(cx + sz, cy - 2 * sz);
				glVertex3d(cx + sz, 0, cy - sz);
				glTexCoord2d(cx - sz, cy - 2 * sz);
				glVertex3d(cx - sz, 0, cy - sz);

				glNormal3d(0, 0, 1);
				glTexCoord2d(cx + sz, cy + sz);
				glVertex3d(cx + sz, h, cy + sz);
				glTexCoord2d(cx - sz, cy + sz);
				glVertex3d(cx - sz, h, cy + sz);
				glTexCoord2d(cx - sz, cy + 2 * sz);
				glVertex3d(cx - sz, 0, cy + sz);
				glTexCoord2d(cx + sz, cy + 2 * sz);
				glVertex3d(cx + sz, 0, cy + sz);

				glNormal3d(1, 0, 0);
				glTexCoord2d(cx + sz, cy - sz);
				glVertex3d(cx + sz, h, cy - sz);
				glTexCoord2d(cx + sz, cy + sz);
				glVertex3d(cx + sz, h, cy + sz);
				glTexCoord2d(cx + 2 * sz, cy + sz);
				glVertex3d(cx + sz, 0, cy + sz);
				glTexCoord2d(cx + 2 * sz, cy - sz);
				glVertex3d(cx + sz, 0, cy - sz);

				glNormal3d(-1, 0, 0);
				glTexCoord2d(cx - sz, cy + sz);
				glVertex3d(cx - sz, h, cy + sz);
				glTexCoord2d(cx - sz, cy - sz);
				glVertex3d(cx - sz, h, cy - sz);
				glTexCoord2d(cx - 2 * sz, cy - sz);
				glVertex3d(cx - sz, 0, cy - sz);
				glTexCoord2d(cx - 2 * sz, cy + sz);
				glVertex3d(cx - sz, 0, cy + sz);

				glEnd();
			}
		}
	}

	// Draw the earthworm.

	glDisable(GL_TEXTURE_2D);
	glEnable(GL_CULL_FACE);

	fv[0] = .4;
	fv[1] = .3;
	fv[2] = .1;
	fv[3] = 0;
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fv);
	fv[0] = 1;
	fv[1] = 1;
	fv[2] = 1;
	fv[3] = 0;
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, fv);
	fv[0] = 90;
	glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, fv);

	for(i = 0; i < worm.length; i++) {
		double radius = .04;
		int bevel = 10 + worm.length / 10, bevelsize = worm.length / 30;
		double wobblex, wobblez, wobble;
		
		if(i <= 15) {
			radius = .02 + .02 * sin(i * M_PI / 2 / 15);
		} else if(i >= worm.length - 3) {
			radius = .02 + .02 * sin((worm.length - i - 1) * M_PI / 2 / 3);
		}

		if(i >= bevel - bevelsize && i <= bevel + bevelsize) radius += 0.01;

		glPushMatrix();
		wobble = .02 * sin((i + worm.wobblecount) * M_PI / 8);
		if(i) {
			double length;

			wobblex = worm.segs[i].x - worm.segs[i - 1].x;
			wobblez = worm.segs[i].z - worm.segs[i - 1].z;
			length = hypot(wobblex, wobblez);
			wobblex /= length;
			wobblez /= length;
		} else {
			double length;

			wobblex = worm.segs[1].x - worm.segs[0].x;
			wobblez = worm.segs[1].z - worm.segs[0].z;
			length = hypot(wobblex, wobblez);
			wobblex /= length;
			wobblez /= length;
		}
		wobblex *= wobble;
		wobblez *= wobble;
		glTranslated(worm.segs[i].x + wobblex, .01, worm.segs[i].z + wobblez);
		gluSphere(quad, radius, 8, 8);
		glPopMatrix();
	}

	// Draw the flower.

	h = .08 * growtimer / 50;
	size = .07 * growtimer / 50;
	glColor3d(.0, .3, .0);
	glDisable(GL_LIGHTING);
	glLineWidth(5);
	glBegin(GL_LINES);
	glVertex3d(foodpos[0], 0, foodpos[1]);
	glVertex3d(foodpos[0], h, foodpos[1]);
	glEnd();
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, T_FLOWER);
	glDisable(GL_CULL_FACE);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor3d(1, 1, 1);
	glBegin(GL_TRIANGLE_FAN);
	glTexCoord2d(.5, .5);
	glVertex3d(foodpos[0], h, foodpos[1]);
	for(i = 0; i <= 8; i++) {
		glTexCoord2d(.5 + .5 * cos(i * M_PI * 2 / 8), .5 + .5 * sin(i * M_PI * 2 / 8));
		glVertex3d(foodpos[0] + size * cos(i * M_PI * 2 / 8), h * 1.1, foodpos[1] + size * sin(i * M_PI * 2 / 8));
	}
	glEnd();

	// If there should be an explosion, draw it.

	if(gameover) {
		glDisable(GL_DEPTH_TEST);
		glEnable(GL_BLEND);
		glBlendFunc(GL_ONE, GL_ONE);
		glEnable(GL_LIGHTING);
		glDisable(GL_TEXTURE_2D);
		fv[0] = .8;
		fv[1] = .3;
		fv[2] = .1;
		fv[3] = 0;
		glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fv);
		fv[0] = 1;
		fv[1] = .4;
		fv[2] = 0;
		fv[3] = 0;
		glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, fv);
		fv[0] = 90;
		glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, fv);
		for(i = 0; i < NPARTICLE; i++) {
			glPushMatrix();
			glTranslated(blamparticle[i].x, blamparticle[i].y, blamparticle[i].z);
			gluSphere(quad, .03 + (50 - gameover) * .2 / 50, 8, 8);
			glPopMatrix();
		}
	}

	// Draw the static image in the upper left corner.

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-4, +4, -3, +3, -1, 1);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glDisable(GL_LIGHTING);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE, GL_ONE);
	glDisable(GL_CULL_FACE);
	glColor3d(1, 1, 1);

	glBindTexture(GL_TEXTURE_2D, T_HEADER);
	glBegin(GL_QUADS);
	glTexCoord2d(0, 0);
	glVertex3d(-3.7, 2.6, 0);
	glTexCoord2d(1, 0);
	glVertex3d(1, 2.6, 0);
	glTexCoord2d(1, 1);
	glVertex3d(1, 2.1, 0);
	glTexCoord2d(0, 1);
	glVertex3d(-3.7, 2.1, 0);
	glEnd();

	// Draw the current score.

	glBindTexture(GL_TEXTURE_2D, T_FONT);
	glBegin(GL_QUADS);
	i = score / 10;
	glTexCoord2d((double) i / 10, 0);
	glVertex3d(2.6, 2.7, 0);
	glTexCoord2d((double) (i + .93) / 10, 0);
	glVertex3d(3.2, 2.7, 0);
	glTexCoord2d((double) (i + .93) / 10, 1);
	glVertex3d(3.2, 2.1, 0);
	glTexCoord2d((double) i / 10, 1);
	glVertex3d(2.6, 2.1, 0);
	i = score % 10;
	glTexCoord2d((double) i / 10, 0);
	glVertex3d(3.2, 2.7, 0);
	glTexCoord2d((double) (i + .93) / 10, 0);
	glVertex3d(3.8, 2.7, 0);
	glTexCoord2d((double) (i + .93) / 10, 1);
	glVertex3d(3.8, 2.1, 0);
	glTexCoord2d((double) i / 10, 1);
	glVertex3d(3.2, 2.1, 0);
	glEnd();
}

// Check whether a given point collides with a stone block or (possibly,
// depending on "justblocks") by the worm itself. If the point is occupied by
// the worm, the output parameter "hint" is set to the number of the colliding
// segment.

int isfree(double x, double z, int justblocks, int *hint) {
	int xblock, zblock;
	int i;

	if(x <= -2 || x >= 2 || z <= -2 || z >= 2) return 0;

	xblock = (x + 2) * 4;
	zblock = (z + 2) * 4;
	if(board[zblock * 16 + xblock] == '#') return 0;

	if(justblocks) return 1;

	for(i = 20; i < worm.length; i++) {
		double dx, dz, dist;

		dx = x - worm.segs[i].x;
		dz = z - worm.segs[i].z;
		dist = dx * dx + dz * dz;
		if(dist < .01) {
			if(hint) *hint = i;
			return 0;
		}
	}

	return 1;
}

// Enter game-over mode and set up the explosion animation.

void blam(double x, double z) {
	int i;

	gameover = 50;
	for(i = 0; i < NPARTICLE; i++) {
		blamparticle[i].x = x;
		blamparticle[i].y = .02;
		blamparticle[i].z = z;
		blamparticle[i].dx = (double) ((rand() & 127) - 64) * .0002;
		blamparticle[i].dy = (double) ((rand() & 127) - 64) * .0002;
		blamparticle[i].dz = (double) ((rand() & 127) - 64) * .0002;
	}
}

// Place the flower at a random location.

void placefood() {
	do {
		foodpos[0] = (double) ((rand() & 16383) - 8192) / 4096;
		foodpos[1] = (double) ((rand() & 16383) - 8192) / 4096;
	} while(
		!isfree(foodpos[0], foodpos[1], 0, 0) ||
		!isfree(foodpos[0] + .1, foodpos[1], 0, 0) ||
		!isfree(foodpos[0] - .1, foodpos[1], 0, 0) ||
		!isfree(foodpos[0], foodpos[1] + .1, 0, 0) ||
		!isfree(foodpos[0], foodpos[1] - .1, 0, 0));
	growtimer = 0;
}

// Reset the game state.

void newgame() {
	int i;

	memset(&worm, 0, sizeof(worm));
	worm.length = 30;
	worm.growto = 30;
	worm.speed = 0.018;
	for(i = 0; i < worm.length; i++) {
		worm.segs[i].x = .5 - worm.speed * i;
		worm.segs[i].z = 0;
	}
	score = 0;
	placefood();
}

// Clock the game state machine once.

void rungame(int rotation, int uppressed) {
	double dx, dz, side[2];
	double speed;
	double length, dist;
	int i;

	if(gameover) {
		// Run the explosion physics.

		for(i = 0; i < NPARTICLE; i++) {
			blamparticle[i].x += blamparticle[i].dx;
			blamparticle[i].y += blamparticle[i].dy;
			blamparticle[i].z += blamparticle[i].dz;
			if(blamparticle[i].y < .01) blamparticle[i].y = .01;
			blamparticle[i].dy -= .0005;
		}
		gameover--;
		if(!gameover) {
			newgame();
		}

		// Since we're in game-over state, there's nothing more for rungame() to do.

		return;
	}

	// Animate the flower and do the worm wobbling.

	if(growtimer < 50) growtimer++;
	worm.wobblecount++;

	// Change the current direction according to user input.

	worm.angle += rotation;

	// Move the worm.

	speed = (uppressed? 1.9 : 1) * worm.speed;
	dx = speed * cos(worm.angle * M_PI / 180);
	dz = speed * sin(worm.angle * M_PI / 180);

	if(worm.growto > worm.length) worm.length++;
	for(i = worm.length - 1; i; i--) {
		memcpy(&worm.segs[i], &worm.segs[i - 1], sizeof(struct segment));
	}

	worm.segs[0].x += dx;
	worm.segs[0].z += dz;

	// Check for collisions.

	side[0] = dz;
	side[1] = -dx;
	normalize2(side);
	side[0] *= .04;
	side[1] *= .04;

	if(!isfree(worm.segs[0].x, worm.segs[0].z, 0, 0)) {
		blam(worm.segs[0].x, worm.segs[0].z);
	} else if(!isfree(worm.segs[0].x + side[0], worm.segs[0].z + side[1], 1, &i)) {
		blam(worm.segs[i].x, worm.segs[i].z);
	} else if(!isfree(worm.segs[0].x - side[0], worm.segs[0].z - side[1], 1, &i)) {
		blam(worm.segs[i].x, worm.segs[i].z);
	}

	// Did the worm hit the flower?

	dx = worm.segs[0].x - foodpos[0];
	dz = worm.segs[0].z - foodpos[1];
	dist = dx * dx + dz * dz;
	if(dist < .004) {
		// Yes! Increase score and difficulty.

		score++;
		if(score > 99) score = 99;

		worm.growto += 5;
		if(worm.growto > MAXSEG) worm.growto = MAXSEG;
		worm.speed += .0005;

		placefood();
	}
}

// A recurrent timer callback from SDL, which generates an event. The event
// typically causes SDL_WaitEvent in the main loop to return.

Uint32 timercallback(Uint32 interval, void *dummy) {
	SDL_Event event;

	event.type = SDL_USEREVENT;
	SDL_PushEvent(&event);

	return interval;
}

// The entry point of the program.

int main(int argc, char **argv) {
	int i, x, y;
	int leftpressed = 0, rightpressed = 0, uppressed = 0, rotation = 0;
	unsigned char buf[64*64*4];
	int fullscreen = 0;

	if(argc == 2 && !strcmp(argv[1], "-f")) fullscreen = 1;

	if(SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO)) {
		err(1, "SDL_Init");
	}

	atexit(SDL_Quit);

	if(!SDL_SetVideoMode(WIDTH, HEIGHT, 16, SDL_HWSURFACE | SDL_OPENGL | (fullscreen? SDL_FULLSCREEN : 0))) {
		err(1, "SDL_SetVideoMode");
	}

	SDL_ShowCursor(0);

	// Initiate the game status.

	newgame();

	// Initiate OpenGL and load the textures.

	quad = gluNewQuadric();
	if(!quad) err(1, "gluNewQuadric");
	gluQuadricDrawStyle(quad, GLU_FILL);

	glBindTexture(GL_TEXTURE_2D, T_GROUND);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, soil_WIDTH, soil_HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, soil_data);
	glBindTexture(GL_TEXTURE_2D, T_STONES);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, stones_WIDTH, stones_HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, stones_data);
	glBindTexture(GL_TEXTURE_2D, T_FONT);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, font_WIDTH, font_HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, font_data);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glBindTexture(GL_TEXTURE_2D, T_HEADER);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, header_WIDTH, header_HEIGHT, GL_RGB, GL_UNSIGNED_BYTE, header_data);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);

	// The flower texture needs an alpha component, but the pixel data
	// doesn't contain it. This code creates a local copy of the graphics
	// data, in which every black pixel is transparent.

	for(y = 0; y < 64; y++) {
		for(x = 0; x < 64; x++) {
			unsigned char r, g, b, a;

			r = flower_data[y * 64 * 3 + x * 3 + 0];
			g = flower_data[y * 64 * 3 + x * 3 + 1];
			b = flower_data[y * 64 * 3 + x * 3 + 2];
			a = 255;
			if(r == 0 && g == 0 && b == 0) a = 0;
			buf[y * 64 * 4 + x * 4 + 0] = r;
			buf[y * 64 * 4 + x * 4 + 1] = g;
			buf[y * 64 * 4 + x * 4 + 2] = b;
			buf[y * 64 * 4 + x * 4 + 3] = a;
		}
	}
	glBindTexture(GL_TEXTURE_2D, T_FLOWER);
	gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGBA, 64, 64, GL_RGBA, GL_UNSIGNED_BYTE, buf);

	// Start a recurrent 20 ms timer which will generate SDL_USEREVENT
	// events, which will eventually cause rungame() to be called at 50 Hz.

	SDL_AddTimer(20, timercallback, 0);

	for(;;) {
		SDL_Event event;

		while(SDL_PollEvent(&event)) {
			switch(event.type) {
				case SDL_USEREVENT:
					rungame(rotation * 5, uppressed);
					break;
				case SDL_KEYUP:
					switch(event.key.keysym.sym) {
						case SDLK_LEFT:
							leftpressed = 0;
							if(rightpressed) {
								rotation = 1;
							} else {
								rotation = 0;
							}
							break;
						case SDLK_RIGHT:
							rightpressed = 0;
							if(leftpressed) {
								rotation = -1;
							} else {
								rotation = 0;
							}
							break;
						case SDLK_UP:
							uppressed = 0;
							break;
					}
					break;
				case SDL_KEYDOWN:
					switch(event.key.keysym.sym) {
						case SDLK_LEFT:
							leftpressed = 1;
							rotation = -1;
							break;
						case SDLK_RIGHT:
							rightpressed = 1;
							rotation = 1;
							break;
						case SDLK_UP:
							uppressed = 1;
							break;
						case SDLK_ESCAPE:
							exit(0);
							break;
					}
					break;
			}
		}
		drawframe();
		SDL_GL_SwapBuffers();
	}

	return 0;
}
